# Copyright © 2008 Paul Gausden. All rights reserved. 

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

require 'sketchup.rb'

$DICT="MSTS1"
$LOD="LOD"
$LODS="LODLIST"

if( not file_loaded?("TSScripts.rb") )

    # This will add a separator to the menu, but only once
    add_separator_to_menu($exStrings.GetString("Plugins"))
    
    # add sub-menu
    tmenu = UI.menu($exStrings.GetString("Plugins")).add_submenu($exStrings.GetString("Train Simulator"))
    
    tmenu.add_item($exStrings.GetString("MS Train Sim eXport")) { exportSFile Sketchup.active_model }
    
    tmenu.add_item($exStrings.GetString("Define LODs for Current Model")) { defLODs Sketchup.active_model }
    tmenu.add_item($exStrings.GetString("Highlight Untextured")) { HighlightUntextured() }
    
    #tmenu.add_item($exStrings.GetString("Show Parents")) { showparent }
    
    path = Sketchup.find_support_files "help.htm", "Plugins/TS/"
    
    tmenu.add_item($exStrings.GetString("Help")) { UI.openURL("file://" + path.to_s) } if path
    
	UI.add_context_menu_handler do |menu|
		menu.add_separator
		$mnulod=menu.add_item("MSTS LOD") { setLOD }
	end
  
  # ==========================================================================
  # ============================ Add Toolbar Menu ============================
  # 
  # Copyleft (2016) Stingray
  # Mail: <chsgate@seznam.cz>
  # Web: http://pokolejich.cz/msts
  # 
  # Create toolbar
  tstb = UI::Toolbar.new("MSTS Toolbar")
  
  # Add MSTS Export button
  tscmd = UI::Command.new("MS Train Sim eXport") { exportSFile Sketchup.active_model }
  tscmd.large_icon = "./images/TS01_32.png"
  tscmd.small_icon = "./images/TS01_16.png"
  tscmd.status_bar_text = tscmd.tooltip = "Exports current model to Shape file"
  tstb.add_item tscmd
  
  # Add MSTS Export button
  tscmd = UI::Command.new("Define LODs for Current Model") { defLODs Sketchup.active_model }
  tscmd.large_icon = "./images/TS02_32.png"
  tscmd.small_icon = "./images/TS02_16.png"
  tscmd.status_bar_text = tscmd.tooltip = "Define LODs for Current Model"
  tstb.add_item tscmd

  # Add MSTS Export button
  tscmd = UI::Command.new("Highlight Untextured") { HighlightUntextured() }
  tscmd.large_icon = "./images/TS03_32.png"
  tscmd.small_icon = "./images/TS03_16.png"
  tscmd.status_bar_text = tscmd.tooltip = "Highlight Untextured Faces"
  tstb.add_item tscmd

  # Show toolbar
  tstb.show
  # END of creating toolbar
  # ==========================================================================

end
#-----------------------------------------------------------------------------
file_loaded("TSScripts.rb")

#-----------------------------------------------------------------------------
# Export.rb converts the model to msts S file format

Sketchup.require 'TS/mstsx'

####################################################################
# run the export (from menu)
# select the file name and process the model
def exportSFile(model)
	# select the output file
	fName=UI.savepanel $exStrings.GetString("Save Model"),"",model.title + ".S"
	
	# if not cancelled
	if fName
		# create file object and scan the model
		tmFile=SFile.new(model, fName)
		
		# if the initialize opened the new file correctly
		if tmFile.fh
			tmFile.saveS
			
			if not FileTest.exist?(fName + "d")
				key=UI.messagebox $exStrings.GetString("Make default SD file?"), MB_YESNO,"SD Save"
				if key==6  # yes
					tmFile.saveSD
				end
			end
			UI.messagebox tmFile.message + "\n" + $exStrings.GetString("Export finished ") + tmFile.fileName , MB_OK, "S Save"
		else
			UI.messagebox $exStrings.GetString("Failed to open ") + tmFile.fileName , MB_OK, "Error"
		end
	end
	# end of export
end

##################################################################

def defLODs(model)
	dlg = UI::WebDialog.new("MSTS Model LOD Setup", true, "MSTSLod", 350, 250, 150, 150, true);
	html = File.dirname(__FILE__) + "/lods.html";
	if(html.length == 0)
		return false;
	end
	dlg.set_file(html, nil);

	dlg.add_action_callback("doSave") { |d,p|
		model.lods = p.split("/")
		d.close }

	dlg.add_action_callback("onStartup") { |d,p|
		model.lods.each { |i|
			d.execute_script("addValue(lodlist.options,#{i.to_s})")
		}
	}
	dlg.add_action_callback("clear") { |d,p|
		model.lods=[2000]
		model.entities.each { |e| e.lod=nil }
	}
	dlg.add_action_callback("endit") { |d,p| d.close }
	dlg.show_modal
end

def setLOD()
	model=Sketchup.active_model
	ss = model.selection
	if ss.count>=1 then
		lastLod=nil
		ss.each { |e|
			if lastLod then
				if lastLod!=e.lod
					lastLod=nil
					break
				end
			else
				lastLod=e.lod
			end
		}

		dlg = UI::WebDialog.new("MSTS Model LOD Selection", true, "MSTSLodSel", 350, 250, 150, 150, true);
		html = File.dirname(__FILE__) + "/setlod.html";
		if(html.length == 0)
			return false;
		end
		dlg.set_file(html, nil);
		dlg.add_action_callback("doSave") { |d,p|
			ss.each { |e|
				e.lod=p
			}
			d.close }
		dlg.add_action_callback("onStartup") { |d,p|
			model.lods.each { |i|
				d.execute_script("addValue(lodlist.options,#{i.to_s})")
			}
			if lastLod then
				d.execute_script("lodlist.value='#{lastLod}'")
			end
		}
		dlg.add_action_callback("endit") { |d,p| d.close }
		dlg.show_modal
	end
end
####################################################################
# add == to color
class Sketchup::Color
	 def ==(c)
		 return (c.alpha==self.alpha and c.red==self.red and c.green==self.green and c.blue==self.blue)
	 end
 end

class Sketchup::Entity

	attr_accessor :groupId
	#private :adjustLods
	
	def lod
		# get lod if defined
		ilod=get_attribute($DICT,$LOD,nil)
		# return value
		return ilod.to_i if ilod
		# end recurse
		return 0 if parent==model
		# 
		return parent.lod
	end
	
	# set lod property
	def lod=(val)
		if (typename=="Face") then
			set_attribute($DICT,$LOD,val)
		elsif (typename=="ComponentInstance") then
			set_attribute($DICT,$LOD,val)
			adjustLods definition.entities,val		
		elsif (typename=="Group" or typename=="ComponentDefinition") then
			set_attribute($DICT,$LOD,val)
			adjustLods entities,val
		end
	end
	
	# make sure no children have higher values
	def adjustLods(ents,val)
		ents.each do |e|
			if val then
				ilod=e.get_attribute($DICT,$LOD,0).to_i
				if ilod>val.to_i then
					# this continues the recursion
					e.lod=val.to_i
				elsif (e.typename=="ComponentDefinition" or e.typename=="Group")
					adjustLods e.entities,val
				end
			else
				# recursive clear
				e.lod=nil
			end
		end
	end
end

class Sketchup::Model
	attr_accessor :groupId
	
	def lods=(val)
		val.collect! do |x|
      x =x.to_s + "/"
    end
    
    #-------------------------------------------------------------------------
    # I'm sorry for doing that, but this plugin is (or may be) abandoned since
    # 2008 and author is not working on it. Plugin was functional for the last
    # time in SketchUp 2013. Starting with SU 2014 LODs generating was broken.
    # I'm not Ruby programmer, but I found this error and repaired it. 
    # 
    # I fully respect author's copyright. But we need this plugin for creating
    # some models in SU and we can use current SU version, not archive...
    # (2016) Stingray
    # Mail: <chsgate@seznam.cz>
    # Web: http://pokolejich.cz/msts
    #-------------------------------------------------------------------------
		## BAD! ==> set_attribute($DICT,$LODS,val.to_s)
    ## GOOD: v-v-v
		set_attribute($DICT,$LODS,val.join)
    ## That's all, folks! ;-)
	end
	
	def lods
		return get_attribute($DICT,$LODS,"2000/").split("/")
	end
	
end

#-----------------------------------------------------------------------------
# with apologies for hacking about this code to....
# Copyright (c) 2006,2007 Jonathan Harris
# 
# Mail: <x-plane@marginal.org.uk>
# Web:  http://marginal.org.uk/x-planescenery/
#
# This software is licensed under a Creative Commons
#   Attribution-Noncommercial-ShareAlike license:
#   http://creativecommons.org/licenses/by-nc-sa/3.0/

def HighlightUntextured()

  model=Sketchup.active_model
  materials=model.materials
  model.start_operation("Highlight Untextured")
  begin
    untextured=materials["Untextured"]
    if (not untextured) or (untextured.texture and untextured.texture.filename)
	untextured=materials.add("Untextured")
	untextured.color="Red"
	untextured.alpha=1.0
	untextured.texture=nil
    end
    
    count=GetHighlightFaces(model.entities, untextured)
    model.commit_operation

    # ==========================================================================
    # ================== Display number of untextured faces ====================
    # 
    # Copyleft (2016) Stingray
    # Mail: <chsgate@seznam.cz>
    # Web: http://pokolejich.cz/msts
    # 
    # Display number of untextured faces
    
    case
      when count < 0
        UI.messagebox("ERROR! Counted faces can not be less then 0!")
      when count == 0
        UI.messagebox("No faces to texture.", MB_OK)
      when count > 0
        UI.messagebox("Untextured faces: " + count.to_s + ".", MB_OK)
    end    
    ## Orig --> UI.messagebox "All faces are textured", MB_OK,"X-Plane export" if count==0
    #
	# END
    # ==========================================================================

  rescue
    model.abort_operation
  end

end

def GetHighlightFaces(entities, untextured)

  count=0

  entities.each do |ent|

    case ent.typename

    when "ComponentInstance"
      count+=GetHighlightFaces(ent.definition.entities, untextured)

    when "Group"
      count+=GetHighlightFaces(ent.entities, untextured)

    when "Face"
      if not (ent.material and ent.material.texture and ent.material.texture.filename)  and not (ent.back_material and ent.back_material.texture and ent.back_material.texture.filename)
	ent.material=untextured
	#ent.back_material=reverse
	count+=1
      #else
	#ent.material=reverse if not (ent.material and ent.material.texture and ent.material.texture.filename)
	#ent.back_material=reverse if not (ent.back_material and ent.back_material.texture and ent.back_material.texture.filename)
      end

    end
  end

  return count

end
